local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- the Interface module
local Interface = Addon:NewModule("Interface")

-- local functions
local string_format = string.format
local tinsert       = table.insert
local table_sort    = table.sort
local pairs         = pairs

-- aux variables
local _

local monitored = {}

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- local constants
local MAX_LIST_SIZE       = 10
local LIST_ENTRY_HEIGHT   = 20

local FILTER_INSTANCE_OPT_ALL = "Show All"

-- sort function for list entries (instance, then name)
local function dataLessThan(a, b) 
	return a.localized == b.localized and a.name < b.name or a.localized < b.localized
end

local filterInstanceBtnInfo = { func = function(self) Interface:HandleInstanceFilter(self.value); UIDropDownMenu_SetSelectedValue(Interface.logFilterInstance, self.value) end } 

local function InitInstanceFilter()
	if Addon:InCombat() then
		return
	end
	
	filterInstanceBtnInfo.text = L["Show All"]
	filterInstanceBtnInfo.value = FILTER_INSTANCE_OPT_ALL
	filterInstanceBtnInfo.checked = Interface.instanceFilter == FILTER_INSTANCE_OPT_ALL
	UIDropDownMenu_AddButton(filterInstanceBtnInfo)
	
	-- populate with currently monitored instances
	NS:ClearTable(monitored)
	
	for instance, _ in Addon:IterateMonitoredInstances() do
		tinsert(monitored, instance)
	end
	
	table_sort(monitored, function(a, b) return NS:TranslateInstance(a) < NS:TranslateInstance(b); end)
	
	for i = 1, #monitored do
		filterInstanceBtnInfo.text = NS:TranslateInstance(monitored[i])
		filterInstanceBtnInfo.value = monitored[i]
		filterInstanceBtnInfo.checked = Interface.instanceFilter == monitored[i]
		UIDropDownMenu_AddButton(filterInstanceBtnInfo)
	end

	UIDropDownMenu_SetSelectedValue(Interface.logFilterInstance, Interface.instanceFilter)
end

local function MatchListUpdate()
	local frame = Interface.dataListFrame
	local entries = frame.entries
	local numEntries = #Interface.filteredEntries
	local listSize = min(numEntries, MAX_LIST_SIZE)

	local scrollFrame = frame.scrollFrame
	
	local offset = FauxScrollFrame_GetOffset(scrollFrame)
	FauxScrollFrame_Update(scrollFrame, numEntries, listSize, LIST_ENTRY_HEIGHT)

	-- if the list gets shortened so much that all entries are below the current offset then adjust the offset
	if offset + MAX_LIST_SIZE > numEntries then
		offset = numEntries - MAX_LIST_SIZE
		if offset < 0 then
			offset = 0
		end
		FauxScrollFrame_SetOffset(scrollFrame, offset)
	end	
	
	local timeOffset = Addon:GetTimeOffset()
	for i = 1, MAX_LIST_SIZE do
		local index = i + offset
		local entry = entries[i]

		if index <= numEntries then
			local data = Interface.filteredEntries[index]
			entry.instanceFrame.text:SetText(data.localized)
			entry.myraidFrame.text:SetText(Addon:IsInRaid(data.name) and "*" or "")
			entry.characterFrame.text:SetText(Addon:ColorizeChar(tostring(data.name)))
			entry.levelFrame.text:SetText(tostring(data.level))
			entry.guildFrame.text:SetText(tostring(data.guild))
			
			entry.data = data
			
			if Interface.selectedEntry == data then
				entry:SetBackdropColor(1, 1, 1, 1)
			else
				entry:SetBackdropColor(0, 0, 0, .33)
			end
			
			entry:Show()
		else
			entry:Hide()
		end
		
		if scrollFrame:IsShown() then
			entry:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -20, -1*(i-1)*LIST_ENTRY_HEIGHT - 3)
		else
			entry:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -3, -1*(i-1)*LIST_ENTRY_HEIGHT - 3)
		end		
	end
end

-- module handling
function Interface:OnInitialize()
	self.filteredEntries  = {}

	self:Setup()
end

function Interface:OnEnable()
	-- empty
end

function Interface:OnDisable()
	-- empty
end

-- log window
function Interface:Setup()
	-- log window setup
	local backdrop = {
		-- path to the background texture
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", 
		-- path to the border texture
		edgeFile = "Interface\\TutorialFrame\\TutorialFrameBorder",
		-- true to repeat the background texture to fill the frame, false to scale it
		tile = true,
		-- size (width or height) of the square repeating background tiles (in pixels)
		tileSize = 32,
		-- thickness of edge segments and square size of edge corners (in pixels)
		edgeSize = 28,
		-- distance from the edges of the frame to those of the background texture (in pixels)
		insets = {
			left   = 5,
			right  = 2,
			top    = 20,
			bottom = 5
		}
	}
	
	self.logWindow = CreateFrame("Frame", ADDON.."LogWindow", UIParent)
	self.logWindow:SetFrameStrata("DIALOG")
	self.logWindow:CreateTitleRegion():SetAllPoints()
	self.logWindow:SetToplevel(true)
	self.logWindow:EnableMouse(true)
	self.logWindow:SetMovable(true)
	self.logWindow:SetUserPlaced(true)
	self.logWindow:SetHeight(370)
	self.logWindow:SetWidth(500)
	self.logWindow:SetBackdrop(backdrop)
	self.logWindow:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	self.logWindow:RegisterForDrag("LeftButton")
	
	tinsert(UISpecialFrames, self.logWindow:GetName())
	
	local titleBar = self.logWindow:CreateTitleRegion()
	titleBar:SetPoint("TOPRIGHT", -5, 0)
	titleBar:SetPoint("TOPLEFT", 5, 0)
	titleBar:SetHeight(28)

	self.logWindow.title = self.logWindow:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
	self.logWindow.title:SetPoint("CENTER", titleBar, "CENTER", 0, 8)
	self.logWindow.title:SetText(Addon.FULLNAME .. " - " .. L["Log Window"])
	
	local button = CreateFrame("Button", nil, self.logWindow, "UIPanelCloseButton")
	button:SetPoint("TOPRIGHT", 5, 6)
	
	-- label for instance filter
	local frame = CreateFrame("Frame", nil, self.logWindow)
	frame:SetHeight(20)
	frame:SetWidth(85)
	frame:SetPoint("TOPLEFT", self.logWindow, "TOPLEFT", 15, -35)
	
	local text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	text:SetPoint("LEFT", frame, "LEFT")
	text:SetText(L["Show Instance:"])
	
	-- combobox for instance filter
	self.instanceFilter = FILTER_INSTANCE_OPT_ALL
	
	self.logFilterInstance = CreateFrame("Frame", ADDON.."InstanceFilter", self.logWindow, "UIDropDownMenuTemplate")
	
	self.logFilterInstance:ClearAllPoints()
	self.logFilterInstance:SetPoint("TOPLEFT", self.logWindow, "TOPLEFT", 100, -30)
	
	UIDropDownMenu_Initialize(self.logFilterInstance, InitInstanceFilter)
	UIDropDownMenu_SetWidth(self.logFilterInstance, 100);
	UIDropDownMenu_SetButtonWidth(self.logFilterInstance, 124)
	UIDropDownMenu_JustifyText(self.logFilterInstance, "LEFT")
 
	-- THE list
	backdrop = {
		-- path to the background texture
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background-Dark", 
		-- path to the border texture
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		-- true to repeat the background texture to fill the frame, false to scale it
		tile = true,
		-- size (width or height) of the square repeating background tiles (in pixels)
		tileSize = 16,
		-- thickness of edge segments and square size of edge corners (in pixels)
		edgeSize = 16,
		-- distance from the edges of the frame to those of the background texture (in pixels)
		insets = {
			left   = 5,
			right  = 5,
			top    = 5,
			bottom = 5
		}
	}
	
	frame = CreateFrame('Frame', self.logWindow:GetName() .. 'FSList', self.logWindow)
	frame:SetHeight(MAX_LIST_SIZE*LIST_ENTRY_HEIGHT+6)
	frame:SetWidth(480)
	frame:SetBackdrop(backdrop)
	frame:SetPoint("TOPLEFT", self.logWindow, "TOPLEFT", 10, -70)

	-- list entries
	frame.entries = {}

	backdrop = {
		-- path to the background texture
		bgFile = "Interface\\QuestFrame\\UI-QuestTitleHighlight", 
		-- true to repeat the background texture to fill the frame, false to scale it
		tile = false,
		-- distance from the edges of the frame to those of the background texture (in pixels)
		insets = {
			left   = 1,
			right  = 1,
			top    = 1,
			bottom = 1
		}
	}
	
	for i=1, MAX_LIST_SIZE do
		local entry = CreateFrame('Button', frame:GetName() .. 'Entry' .. tostring(i), frame)
		entry:SetHeight(LIST_ENTRY_HEIGHT)
		entry:SetPoint("TOPLEFT",  frame, "TOPLEFT",   3, -1*(i-1)*LIST_ENTRY_HEIGHT - 3)
		entry:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -3, -1*(i-1)*LIST_ENTRY_HEIGHT - 3)
		entry:SetBackdrop(backdrop)
		entry:SetBackdropColor(0, 0, 0, .33)
		-- entry:EnableMouse(true)
		entry:SetScript('OnEnter', function(self) self:SetBackdropColor(1, 1, 1, 1); end)
		entry:SetScript('OnLeave', function(self) if self.data ~= Interface.selectedEntry then self:SetBackdropColor(0, 0, 0, .33); end; end)
		entry:SetScript('OnClick', function(self, button, down) Interface:SetSelectedEntry(self) end)
		
		frame.entries[i] = entry
		
		-- subframes holding text for time, author and message
		entry.instanceFrame = CreateFrame('Frame', entry:GetName() .. '1', entry)
		entry.instanceFrame:SetHeight(LIST_ENTRY_HEIGHT)
		entry.instanceFrame:SetWidth(120)
		entry.instanceFrame:SetPoint("TOPLEFT", entry, "TOPLEFT", 5, 0)
		entry.instanceFrame.text = entry.instanceFrame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
		entry.instanceFrame.text:SetAllPoints()
		entry.instanceFrame.text:SetJustifyH("LEFT")

		entry.myraidFrame = CreateFrame('Frame', entry:GetName() .. '1', entry)
		entry.myraidFrame:SetHeight(LIST_ENTRY_HEIGHT)
		entry.myraidFrame:SetWidth(10)
		entry.myraidFrame:SetPoint("TOPLEFT", entry.instanceFrame, "TOPRIGHT", 5, 0)
		entry.myraidFrame.text = entry.myraidFrame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
		entry.myraidFrame.text:SetAllPoints()
		entry.myraidFrame.text:SetJustifyH("LEFT")
		
		entry.characterFrame = CreateFrame('Frame', entry:GetName() .. '1', entry)
		entry.characterFrame:SetHeight(LIST_ENTRY_HEIGHT)
		entry.characterFrame:SetWidth(100)
		entry.characterFrame:SetPoint("TOPLEFT", entry.myraidFrame, "TOPRIGHT", 5, 0)
		entry.characterFrame.text = entry.characterFrame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
		entry.characterFrame.text:SetAllPoints()
		entry.characterFrame.text:SetJustifyH("LEFT")
		
		entry.levelFrame = CreateFrame('Frame', entry:GetName() .. '1', entry)
		entry.levelFrame:SetHeight(LIST_ENTRY_HEIGHT)
		entry.levelFrame:SetWidth(20)
		entry.levelFrame:SetPoint("TOPLEFT", entry.characterFrame, "TOPRIGHT", 5, 0)
		entry.levelFrame.text = entry.levelFrame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
		entry.levelFrame.text:SetAllPoints()
		entry.levelFrame.text:SetJustifyH("LEFT")
		
		entry.guildFrame = CreateFrame('Frame', entry:GetName() .. '1', entry)
		entry.guildFrame:SetHeight(LIST_ENTRY_HEIGHT)
		entry.guildFrame:SetPoint("TOPLEFT", entry.levelFrame, "TOPRIGHT", 5, 0)
		entry.guildFrame:SetPoint("TOPRIGHT", entry, "TOPRIGHT")
		entry.guildFrame.text = entry.guildFrame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
		entry.guildFrame.text:SetAllPoints()
		entry.guildFrame.text:SetJustifyH("LEFT")
	end
	
	-- the scrollframe
	local scroll = CreateFrame('ScrollFrame', frame:GetName() .. 'ScrollFrame', frame, 'FauxScrollFrameTemplate')
	scroll:SetPoint('TOPLEFT', 5, -4)
	scroll:SetPoint('BOTTOMRIGHT', -26, 3)
	scroll:SetScript('OnVerticalScroll', function(self, arg1)
		FauxScrollFrame_OnVerticalScroll(self, arg1, LIST_ENTRY_HEIGHT, function() MatchListUpdate() end)
	end)
	frame.scrollFrame = scroll

	frame:SetScript('OnShow', function(self) MatchListUpdate() end)
	
	self.dataListFrame = frame
	
	-- frame for the whole entry including message
	backdrop = {
		-- path to the background texture
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background-Dark", 
		-- path to the border texture
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		-- true to repeat the background texture to fill the frame, false to scale it
		tile = true,
		-- size (width or height) of the square repeating background tiles (in pixels)
		tileSize = 16,
		-- thickness of edge segments and square size of edge corners (in pixels)
		edgeSize = 16,
		-- distance from the edges of the frame to those of the background texture (in pixels)
		insets = {
			left   = 5,
			right  = 5,
			top    = 5,
			bottom = 5
		}
	}
	
	local frame2 = CreateFrame('Frame', self.logWindow:GetName() .. 'Info', self.logWindow)
	frame2:SetHeight(40)
	frame2:SetWidth(480)
	frame2:SetBackdrop(backdrop)
	frame2:SetPoint("TOPLEFT", self.logWindow, "TOPLEFT", 10, -285)
	
	frame2.text = frame2:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	frame2.text:SetAllPoints()
	frame2.text:SetPoint("TOPLEFT",     frame2, "TOPLEFT",      5, -5)
	frame2.text:SetPoint("BOTTOMRIGHT", frame2, "BOTTOMRIGHT", -5,  5)
	frame2.text:SetJustifyH("LEFT")
	frame2.text:SetJustifyV("TOP")
	frame2.text:SetWordWrap(true)
	frame2.text:SetNonSpaceWrap(false) 
	
	self.infoFrame = frame2
	
	-- whisper button
	local button = CreateFrame("Button", nil, self.logWindow, "UIPanelButtonTemplate")
	button:SetHeight(20)
	button:SetWidth(80)
	button:SetText("Whisper")
	button:SetScript("OnClick", function(self, button, down) Interface:Whisper() end)
	button:SetPoint("TOPLEFT", self.logWindow, "TOPLEFT", 210, -335)
	
	self.whisperButton = button
end

function Interface:ShowLogWindow()
	if not self.logWindow:IsShown() then
		self.logWindow:Show()
	end
end

function Interface:HandleInstanceFilter(value)
	if not value or value == self.instanceFilter then
		return
	end
	
	self.instanceFilter = value
		
	self:UpdateLogList()
end

function Interface:SetSelectedEntry(selected)
	if not selected then
		return
	end
	
	-- deselect old entry
	for i=1, MAX_LIST_SIZE do
		local entry = self.dataListFrame.entries[i]

		if entry.data == self.selectedEntry then
			entry:SetBackdropColor(0, 0, 0, .33)
		end
	end
	
	-- really needed? if we click the frame OnEnter should have colored it properly before anyway
	selected:SetBackdropColor(1, 1, 1, 1)
	
	self.selectedEntry = selected.data
	
	self:UpdateForSelection()
end

function Interface:UpdateForSelection()
	if self.selectedEntry then
		local data = self.selectedEntry
		local count = Addon:GetCharacterCount(data.instance)		
		local text = NS:TranslateInstance(data.instance) .. " - [" .. NS:FormatTime(data.timestamp + Addon:GetTimeOffset()) .. "]: " .. tostring(count) .. " " .. L["players inside instance"]
		
		self.infoFrame.text:SetText(text)
		
		-- enable whisper button
		self.whisperButton:Enable()		
	else
		self.infoFrame.text:SetText("")
		
		-- disable whisper button
		self.whisperButton:Disable()		
	end
end

function Interface:UpdateLogList()
	NS:ClearTable(self.filteredEntries)

	local found = false 
	
	for _, entry in Addon:IterateCharacterData() do					
		-- check instance filter
		local validInstance = false

		if self.instanceFilter == FILTER_INSTANCE_OPT_ALL then
			validInstance = true
		elseif entry.instance == self.instanceFilter then
			validInstance = true
		end
		
		-- insert into filtered matches
		if validInstance then
			tinsert(self.filteredEntries, entry)
			
			if entry == self.selectedEntry then
				found = true
			end
		end
	end
	
	-- sort filtered entries by instance, then name
	table_sort(self.filteredEntries, dataLessThan)
	
	if not found then
		-- reset selection
		self.selectedEntry = nil

		self:UpdateForSelection()
	end
	
	MatchListUpdate()
end

function Interface:GetSelectedCharacter()
	return self.selectedEntry and self.selectedEntry.name
end

function Interface:Whisper()
	return Addon:WhisperCharacter(self:GetSelectedCharacter())
end

-- test
function Interface:Debug(msg)
	Addon:Debug("(Interface) " .. msg)
end
