local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- the Options module
local Options = Addon:NewModule("Options")

-- internal event handling
Options.callbacks = LibStub("CallbackHandler-1.0"):New(Options)

-- local functions
local pairs   = pairs
local tinsert = table.insert

local _

local LibStub   = LibStub

-- config libraries
local AceConfig 		= LibStub:GetLibrary("AceConfig-3.0")
local AceConfigReg 		= LibStub:GetLibrary("AceConfigRegistry-3.0")
local AceConfigDialog	= LibStub:GetLibrary("AceConfigDialog-3.0")

-- translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- local variables
local defaults = {
	profile = {
		MonitoringActive	= true,
		ExcludeSavedRaids	= false,
		NotifyText			= true,
		NotifySound			= false,
		NotificationSound	= 0,
		LookupInterval		= 5,
		HideHint			= false,
        Minimap				= false,
		Monitored           = {},
	},
}

-- local variables
local soundFiles = {
	[0] = "Sound\\interface\\iTellMessage.wav",
	[1] = "Sound\\Spells\\BeastCall.wav",
	[2] = "Sound\\Doodad\\BellTollAlliance.wav",
	[3] = "Sound\\Doodad\\BellTollHorde.wav",
	[4] = "Sound\\Doodad\\BellTollNightElf.wav",
	[5] = "Sound\\Doodad\\BellTollTribal.wav",
	[6] = "Sound\\Doodad\\BlastedLandsLightningbolt01Stand-Bolt.wav",
	[7] = "Sound\\Spells\\Bonk1.wav",
	[8] = "Sound\\Spells\\Bonk2.wav",
	[9] = "Sound\\Spells\\Bonk3.wav",
	[10] = "Sound\\Doodad\\TK_Control_Consoles.wav",
	[11] = "Sound\\interface\\DropOnGround.wav",
	[12] = "Sound\\interface\\DwarfExploration.wav",
	[13] = "Sound\\Doodad\\DwarfHorn.wav",
	[14] = "Sound\\interface\\Error.wav",
	[15] = "Sound\\Doodad\\Firecrackers_ThrownExplode.wav",
	[16] = "Sound\\Doodad\\G_GongTroll01.wav",
	[17] = "Sound\\Events\\GuldanCheers.wav",
	[18] = "Sound\\Doodad\\KharazahnBellToll.wav",
	[19] = "Sound\\Spells\\KillCommand.wav",
	[20] = "Sound\\interface\\MapPing.wav",
	[21] = "Sound\\Doodad\\G_Mortar.wav",
	[22] = "Sound\\interface\\RaidWarning.wav",
	[23] = "Sound\\interface\\ReadyCheck.wav",
	[24] = "Sound\\Doodad\\BE_ScryingOrb_Explode.wav",
	[25] = "Sound\\Doodad\\Sizzle.wav",
	[26] = "Sound\\Spells\\ThrowWater.wav",
	[27] = "Sound\\interface\\UnsheathMetal.wav",
	[28] = "Sound\\interface\\UnsheathShield.wav",
	[29] = "Sound\\Event Sounds\\Wisp\\WispPissed1.wav",
	[30] = "Sound\\Event Sounds\\Wisp\\WispReady1.wav",
	[31] = "Sound\\Event Sounds\\Wisp\\WispWhat1.wav",
	[32] = "Sound\\Event Sounds\\Wisp\\WispYes1.wav",
}

local soundNames = {
	[0] = L["Default"].." (Tell Message)",
	[1] = "Beast Call",
	[2] = "Bell Toll Alliance",
	[3] = "Bell Toll Horde",
	[4] = "Bell Toll Night Elf",
	[5] = "Bell Toll Tribal",
	[6] = "Bolt",
	[7] = "Bonk 1",
	[8] = "Bonk 2",
	[9] = "Bonk 3",
	[10] = "Control Consoles",
	[11] = "Drop On Ground",
	[12] = "Dwarf Exploration",
	[13] = "Dwarf Horn",
	[14] = "Error",
	[15] = "Firecrackers",
	[16] = "Gong Troll",
	[17] = "Guldan Cheers",
	[18] = "Kharazahn Bell Toll",
	[19] = "Kill Command",
	[20] = "Map Ping",
	[21] = "Mortar",
	[22] = "Raid Warning",
	[23] = "Ready Check",
	[24] = "ScryingOrb Explode",
	[25] = "Sizzle",
	[26] = "ThrowWater",
	[27] = "Unsheath Metal",
	[28] = "Unsheath Shield",
	[29] = "Wisp Pissed",
	[30] = "Wisp Ready",
	[31] = "Wisp What",
	[32] = "Wisp Yes",
}

-- module handling
function Options:OnInitialize()
	-- options
	self.options = {}
	
	-- options
	self.db = LibStub:GetLibrary("AceDB-3.0"):New(Addon.MODNAME.."_DB", defaults, "Default")
		
	self:Setup()
		
	-- profile support
	self.options.args.profile = LibStub:GetLibrary("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied",  "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset",   "OnProfileChanged")

	AceConfigReg:RegisterOptionsTable(Addon.FULLNAME, self.options)
	AceConfigDialog:AddToBlizOptions(Addon.FULLNAME)
end

function Options:OnEnable()
	-- empty
end

function Options:OnDisable()
	-- empty
end

function Options:OnProfileChanged(event, database, newProfileKey)
	self.db.profile = database.profile
	
	Addon:OnOptionsReloaded()
end

-- setup
function Options:Setup()	
    self.options = {
		handler = Addon,
		type = 'group',
		args = {
			raids = {
				type = 'group',
				name = L["Raids"],
				desc = L["Set up which raids you will monitor."],
				order = 10,
				args = {
					-- filled dynamically
				},
			},
			monitoring = {
				type = 'group',
				name = L["Monitoring"],
				desc = L["Set up monitoring parmaters for addon."],
				order = 30,
				args = {
					saved = {
						type = 'toggle',
						name = L["Exclude Saved Raids"],
						desc = L["Exclude raids you are currently saved to."],
						order = 2,
						get = function() return self:GetSetting("ExcludeSavedRaids") end,
						set = function() 
							self:ToggleSetting("ExcludeSavedRaids")
						end,
					},
					lookupInterval = {
						type = 'range',
						name = L["Lookup Interval"],
						desc = L["Periodically check instances with set interval. Time in minutes. When set to zero no automatic lookups will be performed."],
						order = 3,
						get = function() return self:GetSetting("LookupInterval") end,
						set = function(info, key) 
							self:SetSetting("LookupInterval", key)
						end,
						min = 0,
						max = self.MAX_TIME_RANGE,
						step = 1,
					},
				},
			},			
			notifications = {
				type = 'group',
				name = L["Notifications"],
				order = 40,
				desc = L["Set up notifications when addon finds characters in previously empty instance."],
				args = {
					text = {
						type = 'toggle',
						name = L["Text Alert"],
						desc = L["Show text message when addon finds characters in previously empty instance."],
						order = 1,
						get = function() return self:GetSetting("NotifyText") end,
						set = function() 
							self:ToggleSetting("NotifyText")
						end,
					},
					sound = {
						type = 'toggle',
						name = L["Sound Alert"],
						desc = L["Play sound when addon finds characters in previously empty monitored instance."],
						order = 2,
						get = function() return self:GetSetting("NotifySound") end,
						set = function() 
							self:ToggleSetting("NotifySound")
						end,
					},
					notificationSound = { 
						type = 'select',
						name = L["Notification Sound"],
						desc = L["Choose sound to be played on notifications."],
						order = 3,
						get = function() return self:GetSetting("NotificationSound") end,	
						set = function(info, key) 
							self:SetSetting("NotificationSound", key)
						end,
						values = soundNames,
					},
					playSound = { 
						type = 'execute',
						name = L["Play Sound"],
						desc = L["Play selected notification sound."],
						order = 4,
						func = function() Addon:PlaySoundFile(self:GetSoundFile(self:GetSetting("NotificationSound"))) end,	
					},
				},
			},
			active = { 
				type = "toggle",
				name = L["Monitoring Active"],
				desc = L["Activate/Deactivate the monitoring."],
	            order = 1,
				get = function() return self:GetSetting("MonitoringActive") end,	
				set = function()
					self:ToggleSetting("MonitoringActive")
				end,
			},
			minimap = {
				type = 'toggle',
				name = L["Minimap Button"],
				desc = L["Show Minimap Button."],
				order = 3,
				get  = function() return self:GetSetting("Minimap") end,
				set  = function()
					self:ToggleSetting("Minimap")
				end,
			},
			hint = {
				type = 'toggle',
				name = L["Hide Hint"],
				desc = L["Hide usage hint in tooltip."],
				order = 4,
				get  = function() return self:GetSetting("HideHint") end,
				set  = function()
					self:ToggleSetting("HideHint") 
				end,
			},
		},
	}
	
	self:InsertRaidOptions()
end

function Options:InsertRaidOptions()
	for extension, instances in pairs(NS.instances) do
		
		local order = extension*100
		
		local localizedExtension = NS:TranslateExpansion(extension)
		
		-- preface raids by extension name
		self.options.args.raids.args[localizedExtension] = {
			type = "header",
			name = NS:Colorize("Orange", localizedExtension),
			order = order,
		}
						
		for index, instance in pairs(instances) do
			local localizedInstance = NS:TranslateInstance(instance)
			
			-- raid monitoring
			self.options.args.raids.args[instance] = {
				type = 'toggle',
				name = localizedInstance,
				desc = L["Monitor"] .. " " .. localizedInstance .. ".",
				order = order+10*index,
				get = function() return self:IsMonitored(instance) end,
				set = function() 
					self:ToggleMonitored(instance)
				end,
			}
		end
	end	
end

-- settings
function Options:ToggleMonitored(instance)
	if not NS:IsValidInstance(instance) then
		return
	end

	if self.db.profile.Monitored[instance] == true then
		self.db.profile.Monitored[instance] = nil
	else
		self.db.profile.Monitored[instance] = true
	end
	
	-- fire event when monitoring changed
	self.callbacks:Fire(ADDON .. "_MONITORING_CHANGED", instance)
end

function Options:IsMonitored(instance)
	return self.db.profile.Monitored[instance] == true
end

function Options:IsAnyInstanceMonitored()
	return next(self.db.profile.Monitored) ~= nil
end

function Options:IterateMonitoredInstances()
	return pairs(self.db.profile.Monitored)
end

-- getter and info
function Options:GetSetting(option)
	return self.db.profile[option]
end

function Options:SetSetting(option, value)
	local current = self:GetSetting(option)

	if current == value then
		return
	end
	
	self.db.profile[option] = value

	-- fire event when setting changed
	self.callbacks:Fire(ADDON .. "_SETTING_CHANGED", option, value, current)
end

function Options:ToggleSetting(option)
	self:SetSetting(option, not self:GetSetting(option) and true or false)
end

function Options:ToggleSettingTrueNil(option)
	self:SetSetting(option, not self:GetSetting(option) and true or nil)
end

-- utility
function Options:GetSoundFile(id)
	return soundFiles[id] or soundFiles[0]
end

-- test
function Options:Debug(msg)
	Addon:Debug("(Options) " .. msg)
end
