local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- the Tooltip module
local Tooltip = Addon:NewModule("Tooltip")

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- tooltip library
local QT = LibStub:GetLibrary("LibQTip-1.0")

-- coloring tools
local Crayon = LibStub:GetLibrary("LibCrayon-3.0")

-- local functions
local pairs             = pairs
local next              = next
local floor             = floor

-- local variables
local _

local tooltip = nil

-- handler functions
local function HandleInstanceCellClicked(cell, instance, button)
	Addon:ToggleMonitored(instance)
	Tooltip:Refresh()
end

-- module handling
function Tooltip:OnInitialize()
	-- empty
end

function Tooltip:OnEnable()
	-- empty
end

function Tooltip:OnDisable()
	self:Remove()
end

function Tooltip:Create(obj)
	if not self:IsEnabled() then
		return
	end

	tooltip = QT:Acquire(ADDON.."TT", 3)
	
	tooltip:Hide()
	tooltip:Clear()
	tooltip:SetScale(1)
		
	self:Draw()

	tooltip:SetAutoHideDelay(.1, obj)
	tooltip:EnableMouse()
	tooltip:SmartAnchorTo(obj)
	tooltip:Show()
end

function Tooltip:Remove()
	if tooltip then
		tooltip:Hide()
		QT:Release(tooltip)
		tooltip = nil
	end
end

function Tooltip:Refresh()
	if not self:IsEnabled() then
		self:Remove()
		return
	end
	
	self:Draw()
	
	tooltip:Show()
end

function Tooltip:Draw()
	if not tooltip then
		return
	end

	tooltip:Hide()
	tooltip:Clear()

	local now = time()
	
	-- addon header
	local lineNum = tooltip:AddHeader(" ")
	tooltip:SetCell(lineNum, 1, NS:Colorize("White", Addon.FULLNAME), "CENTER", tooltip:GetColumnCount())
	
	local ctrlKeyPressed = IsControlKeyDown()
	
	local interval = Addon:GetSetting("LookupInterval") * 60
	
	if interval == 0 then
		interval = NS.Options.MAX_TIME_RANGE
	end
	
	-- monitored raids
	if ctrlKeyPressed or Addon:GetSetting("MonitoringActive") then
		-- for all extensions
		for extension, instances in pairs(NS.instances) do
			local headerPresent = false
			
			-- for all instances
			for _, instance in pairs(instances) do
				local monitored = Addon:IsMonitored(instance)
				local saved = Addon:IsSaved(instance)
				local excluded = Addon:NotMonitoredDueToLock(instance)
				
				if ctrlKeyPressed or monitored then 
					local localizedInstance = NS:TranslateInstance(instance)
					
					if not headerPresent then
						-- set extension name as header
						tooltip:AddLine(" ")
						lineNum = tooltip:AddLine(" ")
						tooltip:SetCell(lineNum, 1, NS:Colorize("Orange", NS:TranslateExpansion(extension)), "LEFT", tooltip:GetColumnCount())
						
						headerPresent = true
					end

					lineNum = tooltip:AddLine(" ")
					
					local data = Addon:GetInstanceData(instance)
					
					if monitored and data then						
						-- number of characters found in instance
						if data.count > 0 then
							-- instance
							if saved then
								tooltip:SetCell(lineNum, 1, NS:Colorize("Red", localizedInstance) .. " ", "LEFT")
							else
								tooltip:SetCell(lineNum, 1, NS:Colorize("Green", localizedInstance) .. " ", "LEFT")
							end
							
							tooltip:SetCell(lineNum, 2, NS:Colorize("White", tostring(data.count)), "LEFT")
							
							-- time since last check
							local timestamp = now - data.timestamp
							local timestring = ""

							if Crayon and interval then
								timestring = "|cff"..Crayon:GetThresholdHexColor(timestamp, interval, interval * 0.75, interval * 0.5, interval * 0.25, 1) .. NS:FormatTime(timestamp) .. "|r"
							else
								timestring = NS:FormatTime(timestamp)
							end				
							
							tooltip:SetCell(lineNum, 3, timestring, "RIGHT")
						else
							-- instance
							if saved then
								tooltip:SetCell(lineNum, 1, NS:Colorize("Red", localizedInstance) .. " ", "LEFT")
							else
								tooltip:SetCell(lineNum, 1, NS:Colorize("Blueish", localizedInstance) .. " ", "LEFT")
							end
							
							-- no character
							tooltip:SetCell(lineNum, 2, NS:Colorize("GrayOut", "--"), "LEFT")
							
							-- no time
							tooltip:SetCell(lineNum, 3, NS:Colorize("GrayOut", "--"), "RIGHT")
						end
					else
						-- instance
						if excluded then
							tooltip:SetCell(lineNum, 1, NS:Colorize("Yellow", localizedInstance) .. " ", "LEFT")
						elseif saved then
							tooltip:SetCell(lineNum, 1, NS:Colorize("Red", localizedInstance) .. " ", "LEFT")
						else
							tooltip:SetCell(lineNum, 1, NS:Colorize("GrayOut", localizedInstance) .. " ", "LEFT")
						end
						
						-- no character
						tooltip:SetCell(lineNum, 2, NS:Colorize("GrayOut", "--"), "LEFT")
						
						-- no time
						tooltip:SetCell(lineNum, 3, NS:Colorize("GrayOut", "--"), "RIGHT")
					end
					
					tooltip:SetCellScript(lineNum, 1, "OnMouseDown", HandleInstanceCellClicked, instance)					
				end			
			end	
		end
		
	else
		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("Red", L["Monitoring is switched off."]), "LEFT", tooltip:GetColumnCount())
	end
		
	-- hint to show options
	if not Addon:GetSetting("HideHint") then
		tooltip:AddLine(" ")
		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("Brownish", L["Left-Click"] .. ":") .. " " .. NS:Colorize("Yellow", L["Show logging window."]), "LEFT", tooltip:GetColumnCount())
		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("Brownish", L["Alt-Click"] .. ":") .. " " .. NS:Colorize("Yellow", L["Enable/disable monitoring."]), "LEFT", tooltip:GetColumnCount())
		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("Brownish", L["Ctrl-Click"] .. ":") .. " " .. NS:Colorize("Yellow", L["Request data for monitored instances."]), "LEFT", tooltip:GetColumnCount())
		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("Brownish", L["Right-Click"] .. ":") .. " " .. NS:Colorize("Yellow", L["Open option menu."]), "LEFT", tooltip:GetColumnCount())
		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("Brownish", L["Left-Click Tip"] .. ":") .. " " .. NS:Colorize("Yellow", L["Click on instance name to toggle monitoring."]), "LEFT", tooltip:GetColumnCount())
		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("Brownish", L["Ctrl-Hover Tip"] .. ":") .. " " .. NS:Colorize("Yellow", L["Press Ctrl while opening tooltip shows all instances."]), "LEFT", tooltip:GetColumnCount())
	end	
end

-- test
function Tooltip:Debug(msg)
	Addon:Debug("(Tooltip) " .. msg)
end
